@@ -1,15 +1,6 @@
-import os
-from pathlib import Path as _Path
-import json as _json
-import time as _time
-import pandas as _pd
 import gzip
-import typing
 import aanalytics2
-import requests as _requests
-from aanalytics2 import config
-from copy import deepcopy
-from concurrent import futures as _futures
+from aanalytics2 import config, modules, connector
 
 
 class DIAPI:
@@ -43,7 +34,7 @@ class DIAPI:
             path = pkg_resources.resource_filename(
                 "aanalytics2", "supported_tags.pickle")
         with path as f:
-            self.REFERENCE = _pd.read_pickle(f)
+            self.REFERENCE = modules.pd.read_pickle(f)
 
     def getMethod(self, pageName: str = None, g: str = None, pe: str = None, pev1: str = None, pev2: str = None, events: str = None, **kwargs):
         """
@@ -67,7 +58,7 @@ class DIAPI:
         endpoint = f"https://{self.tracking_server}/b/ss/{self.rsid}/0"
         params = {"pageName": pageName, "g": g,
                   "pe": pe, "pev1": pev1, "pev2": pev2, "events": events, **kwargs}
-        res = _requests.get(endpoint, params=params, headers=header)
+        res = modules.requests.get(endpoint, params=params, headers=header)
         return res
 
     def postMethod(self, pageName: str = None, pageURL: str = None, linkType: str = None, linkURL: str = None, linkName: str = None, events: str = None, **kwargs):
@@ -96,7 +87,7 @@ class DIAPI:
         myxml = dxml.dicttoxml(
             dictionary, custom_root='request', attr_type=False)
         xml_data = myxml.decode()
-        res = _requests.post(endpoint, data=xml_data, headers=header)
+        res = modules.requests.post(endpoint, data=xml_data, headers=header)
         return res
 
 
@@ -110,7 +101,13 @@ class Bulkapi:
         endpoint : OPTIONAL : by default using https://analytics-collection.adobe.io
     """
 
-    def __init__(self, endpoint: str = "https://analytics-collection.adobe.io"):
+    def __init__(self, endpoint: str = "https://analytics-collection.adobe.io", config_object: dict = config.config_object):
+        """
+        Initialize the Bulk API connection. Returns an object with methods to send data to Analytics.
+        Arguments:
+            endpoint : REQUIRED : Endpoint to send data to. Default to analytics-collection.adobe.io
+            config_object : REQUIRED : config object containing the different information to send data.
+        """
         self.endpoint = endpoint
         try:
             import importlib.resources as pkg_resources
@@ -122,19 +119,16 @@ class Bulkapi:
             path = pkg_resources.resource_filename(
                 "aanalytics2", "CSV_Column_and_Query_String_Reference.pickle")
         with path as f:
-            self.REFERENCE = _pd.read_pickle(f)
+            self.REFERENCE = modules.pd.read_pickle(f)
         if config.api_key == "":
             raise Exception(
                 "Import config file.\n Authentication is required for this endpoint.")
         # if no token has been generated.
         if len(config.header['Authorization']) < 100:
             token = aanalytics2.retrieveToken()
-        self.clientID = config.api_key
-        self.header = {
-            "Authorization": config.header['Authorization'],
-            "x-api-key": self.clientID,
-            "x-adobe-vgid": "ingestion",
-        }
+        self.connector = connector.AdobeRequest()
+        self.header = self.connector.header
+        self.header['"x-adobe-vgid"'] = "ingestion"
         self._createdFiles = []
 
     def validation(self, file: typing.IO = None, **kwargs):
@@ -159,8 +153,8 @@ class Bulkapi:
             filename = file
             with open(file, "rb") as f:
                 data = f.read()
-        res = _requests.post(self.endpoint+path, files={"file": (None, data)},
-                             headers=self.header)
+        res = modules.requests.post(self.endpoint+path, files={"file": (None, data)},
+                                    headers=self.header)
         return res
 
     def generateTemplate(self, includeAdv: bool = False, returnDF: bool = False, save: bool = True):
@@ -175,7 +169,7 @@ class Bulkapi:
         string = """timestamp,marketingCloudVisitorID,events,pageName,pageURL,reportSuiteID,userAgent,pe,queryString\ntimestampValue,marketingCloudVisitorIDValue,eventsValue,pageNameValue,pageURLValue,reportSuiteIDValue,userAgentValue,peValue,queryStringValue
         """
         data = io.StringIO(string)
-        df = _pd.read_csv(data, sep=',')
+        df = modules.pd.read_csv(data, sep=',')
         if includeAdv == False:
             df.drop(["pe", "queryString"], axis=1, inplace=True)
         if save:
@@ -190,12 +184,12 @@ class Bulkapi:
         if file.endswith(".gz"):
             return file
         else:  # if sending not gzipped file.
-            new_folder = _Path('tmp/')
+            new_folder = modules.Path('tmp/')
             new_folder.mkdir(exist_ok=True)
             with open(file, "rb") as f:
                 content = f.read()
                 new_path = new_folder / f"{file}.gz"
-                with gzip.open(_Path(new_path), 'wb') as f:
+                with gzip.open(modules.Path(new_path), 'wb') as f:
                     f.write(content)
                 # save the filename to delete
                 self._createdFiles.append(new_path)
@@ -227,20 +221,20 @@ class Bulkapi:
         list_headers = [{**self.header, 'x-adobe-vgid': vgid}
                         for vgid in vgid_headers]
         list_urls = [self.endpoint + path for x in range(len(files_gz))]
-        list_files = ({"file": (None, open(_Path(file), "rb").read())}
+        list_files = ({"file": (None, open(modules.Path(file), "rb").read())}
                       for file in files_gz)  # generator for files
         workers_input = kwargs.get("workers", 4)
         workers = max(1, workers_input)
-        with _futures.ThreadPoolExecutor(workers) as executor:
-            res = executor.map(lambda x, y, z: _requests.post(
+        with modules.futures.ThreadPoolExecutor(workers) as executor:
+            res = executor.map(lambda x, y, z: modules.requests.post(
                 x, headers=y, files=z), list_urls, list_headers, list_files)
             list_res = list(res)
         # cleaning temp folder
         if len(self._createdFiles) > 0:
             for file in self._createdFiles:
-                file_path = _Path(file)
+                file_path = modules.Path(file)
                 file_path.unlink()
             self._createdFiles = []
-            tmp = _Path('tmp/')
+            tmp = modules.Path('tmp/')
             tmp.rmdir()
         return list_res
